@@ -6,7 +6,8 @@ from decimal import Decimal
 
 from sql import Null, Column
 
-from trytond.model import ModelView, ModelSQL, Model, UnionMixin, fields
+from trytond.model import (
+    ModelView, ModelSQL, Model, UnionMixin, DeactivableMixin, fields)
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
 from trytond.pool import Pool
@@ -38,7 +39,8 @@ COST_PRICE_METHODS = [
 price_digits = (16, config.getint('product', 'price_decimal', default=4))
 
 
-class Template(ModelSQL, ModelView, CompanyMultiValueMixin):
+class Template(
+        DeactivableMixin, ModelSQL, ModelView, CompanyMultiValueMixin):
     "Product Template"
     __name__ = "product.template"
     name = fields.Char('Name', size=None, required=True, translate=True,
@@ -69,7 +71,6 @@ class Template(ModelSQL, ModelView, CompanyMultiValueMixin):
         fields.Many2One('product.uom.category', 'Default UOM Category'),
         'on_change_with_default_uom_category',
         searcher='search_default_uom_category')
-    active = fields.Boolean('Active', select=True)
     categories = fields.Many2Many('product.template-product.category',
         'template', 'category', 'Categories', states=STATES, depends=DEPENDS)
     categories_all = fields.Many2Many(
@@ -80,25 +81,9 @@ class Template(ModelSQL, ModelView, CompanyMultiValueMixin):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        cursor = Transaction().connection.cursor()
-        sql_table = cls.__table__()
-
         super(Template, cls).__register__(module_name)
 
-        table = TableHandler(cls, module_name)
-        # Migration from 2.2: category is no more required
-        table.not_null_action('category', 'remove')
-
-        # Migration from 2.2: new types
-        cursor.execute(*sql_table.update(
-                columns=[sql_table.consumable],
-                values=[True],
-                where=sql_table.type == 'consumable'))
-        cursor.execute(*sql_table.update(
-                columns=[sql_table.type],
-                values=['goods'],
-                where=sql_table.type.in_(['stockable', 'consumable'])))
+        table = cls.__table_handler__(module_name)
 
         # Migration from 3.8: rename category into categories
         if table.column_exist('category'):
@@ -115,10 +100,6 @@ class Template(ModelSQL, ModelView, CompanyMultiValueMixin):
             return pool.get('product.cost_price_method')
         return super(Template, cls).multivalue_model(field)
 
-    @staticmethod
-    def default_active():
-        return True
-
     @staticmethod
     def default_type():
         return 'goods'
@@ -208,7 +189,8 @@ class TemplateFunction(fields.Function):
         return order
 
 
-class Product(ModelSQL, ModelView, CompanyMultiValueMixin):
+class Product(
+        DeactivableMixin, ModelSQL, ModelView, CompanyMultiValueMixin):
     "Product Variant"
     __name__ = "product.product"
     _order_name = 'rec_name'
@@ -224,7 +206,6 @@ class Product(ModelSQL, ModelView, CompanyMultiValueMixin):
         'product.cost_price', 'product', "Cost Prices")
     description = fields.Text("Description", translate=True, states=STATES,
         depends=DEPENDS)
-    active = fields.Boolean('Active', select=True)
     list_price_uom = fields.Function(fields.Numeric('List Price',
         digits=price_digits), 'get_price_uom')
     cost_price_uom = fields.Function(fields.Numeric('Cost Price',
@@ -259,9 +240,9 @@ class Product(ModelSQL, ModelView, CompanyMultiValueMixin):
                     order_method = TemplateFunction.order(attr)
                     setattr(cls, 'order_%s' % attr, order_method)
 
-    @fields.depends('template')
+    @fields.depends('template', '_parent_template.id')
     def on_change_template(self):
-        for name, field in self._fields.iteritems():
+        for name, field in self._fields.items():
             if isinstance(field, TemplateFunction):
                 if self.template:
                     value = getattr(self.template, name, None)
@@ -309,10 +290,6 @@ class Product(ModelSQL, ModelView, CompanyMultiValueMixin):
         return [product.code] + Template.name.convert_order('name',
             tables['template'], Template)
 
-    @staticmethod
-    def default_active():
-        return True
-
     def get_rec_name(self, name):
         if self.code:
             return '[' + self.code + '] ' + self.name
@@ -458,7 +435,7 @@ class ProductCostPrice(ModelSQL, CompanyValueMixin):
 
         super(ProductCostPrice, cls).__register__(module_name)
 
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         if not exist:
             # Create template column for property migration
             table.add_column('template', 'INTEGER')
